@@ -23,17 +23,20 @@ from munin.provider import \
     GenreTreeProvider, \
     BPMCachedProvider, \
     StemProvider, \
-    WordlistProvider
+    KeywordsProvider
 
 from munin.distance import \
     MoodbarDistance, \
-    GenreTreeDistance, \
-    BPMDistance
+    GenreTreeAvgLinkDistance, \
+    BPMDistance, \
+    KeywordsDistance, \
+    RatingDistance
 
 
 # Checking if the attribute shall be used:
 from munin.provider.moodbar import check_for_moodbar
 from munin.provider.bpm import check_for_bpmtools
+from munin.provider.keywords import check_for_plyr
 
 
 class EasySession(Session):
@@ -41,27 +44,27 @@ class EasySession(Session):
     def from_name(name='EasySession'):
         return Session.from_name(name)
 
-    def __init__(self, name='EasySession'):
+    def __init__(self, name='EasySession', disabled_attrs=None):
 
         mask = {
             'artist': pairup(
                 ArtistNormalizeProvider(compress=True),
                 None,
-                1
+                0.5,
             ),
             'album': pairup(
                 AlbumNormalizeProvider(compress=True),
                 None,
-                1
+                0.5,
             ),
             'title': pairup(
                 TitleNormalizeProvider(compress=False) | StemProvider(),
                 None,
-                1
+                0.5,
             ),
             'genre': pairup(
                 GenreTreeProvider(),
-                GenreTreeDistance(),
+                GenreTreeAvgLinkDistance(),
                 2
             ),
             'bpm': pairup(
@@ -74,10 +77,14 @@ class EasySession(Session):
                 MoodbarDistance(),
                 5
             ),
-            # TODO: Keyword Provider should be used here.
             'lyrics': pairup(
-                WordlistProvider() | StemProvider(),
+                KeywordsProvider(),
+                KeywordsDistance(),
+                3
+            ),
+            'rating': pairup(
                 None,
+                RatingDistance(),
                 3
             )
         }
@@ -90,4 +97,10 @@ class EasySession(Session):
             logging.warning("Disabling bpm attr, no binary found in PATH.")
             del mask['bpm']
 
+        for disabled_attr in disabled_attrs or []:
+            try:
+                del mask[disabled_attr]
+            except KeyError:
+                pass
+
         Session.__init__(self, name, mask)
