@@ -21,9 +21,11 @@ Reference
 ---------
 """
 
+
 # Standard:
-from shutil import rmtree
 from copy import copy
+from shutil import rmtree
+from itertools import islice, tee
 from contextlib import contextmanager
 
 import tarfile
@@ -82,7 +84,7 @@ def get_cache_path(extra_name=None):
 
 
 DEFAULT_CONFIG = {
-    'max_neighbors': 7,
+    'max_neighbors': 10,
     'max_distance': 0.999,
     'history_max_pkg': 10000,
     'history_timeout': 1200,
@@ -95,8 +97,8 @@ DEFAULT_CONFIG = {
     'rebuild_stupid_threshold': 350,
     'recom_history_sieving': True,
     'recom_history_penalty': {
-        'album': 5,
-        'artist': 3
+        'album': 3,
+        'artist': 2
     }
 }
 
@@ -326,13 +328,14 @@ class Session:
             rmtree(path, ignore_errors=True)
         os.mkdir(path)
 
-        class VerbosePickler (pickle._Pickler):
-            def save(self, obj):
-                # print('pickling object  {0} of type {1}'.format(obj, type(obj)))
-                pickle._Pickler.save(self, obj)
+        # class VerbosePickler (pickle._Pickler):
+        #     def save(self, obj):
+        #         print('pickling object  {0} of type {1}'.format(obj, type(obj)))
+        #         pickle._Pickler.save(self, obj)
 
         with open(os.path.join(path, 'session.pickle'), 'wb') as handle:
-            VerbosePickler(handle).dump(self)
+            # VerbosePickler(handle).dump(self)
+            pickle.dump(self, handle)
 
         with tarfile.open(path + '.gz', 'w:gz') as tar:
             tar.add(path, arcname='')
@@ -375,15 +378,30 @@ class Session:
         :return: iterator that yields the sieved songs.
         """
         enabled = self._filtering_enabled
-        given = 0
-
-        for recom in iterator:
-            if number is not 0 and given >= number:
-                break
-
-            if not enabled or self._recom_history.allowed(recom):
+        if not enabled:
+            iterator = islice(iterator, number) if number else iterator
+            for recom in iterator:
                 self._recom_history.feed(recom)
-                given += 1
+                yield recom
+        else:
+            for recom in islice(self._recom_sieve_enabled(iterator), number):
+                yield recom
+
+    def _recom_sieve_enabled(self, iterator):
+            recom_set = set()
+            while True:
+                iterator, starter = tee(iterator)
+                for recom in filter(lambda r: r not in recom_set, starter):
+                    if self._recom_history.allowed(recom):
+                        # we found a candidate!
+                        break
+                else:
+                    # exhausted.
+                    break
+
+                self._recom_history.feed(recom)
+                recom_set.add(recom)
+
                 yield recom
 
     def recommend_from_attributes(self, subset, number=20, max_seeds=10, max_numeric_offset=None):
@@ -405,7 +423,7 @@ class Session:
         return self._recom_sieve(munin.graph.recommendations_from_attributes(
             subset,
             self.database,
-            self.database.rule_index,
+            self.rule_index,
             max_seeds,
             max_numeric_offset
         ), number)
@@ -470,9 +488,12 @@ class Session:
         :retruns: Tuple of the total distance to each other and a list of pairs
                   that consist of (attribute_name: subdistance_float)
         """
+        seed_song = song_or_uid(self.database, seed_song)
+        recommend = song_or_uid(self.database, recommendation)
         return munin.graph.explain_recommendation(
+            self,
             seed_song,
-            recommendation,
+            recommend,
             max_reasons
         )
 
@@ -506,6 +527,42 @@ class Session:
         """
         return self.database.add(value_mapping)
 
+    def modify(self, song, sub_value_mapping):
+        """Modify an existing and known song by the attributes mentioned
+        in ``sub_value_mapping``.
+
+        This should be run during the ``fix_graph`` contextmanager.
+
+        Usage example:
+
+        .. code-block:: python
+
+            >>> with session.fix_graph():
+            ...     session.modify(some_song, {'rating': 5})
+
+        One usecase is shown in the example, the adjustment of the rating.
+
+        .. note::
+
+            This is **not a cheap function**. The song is removed and all
+            distances to it are recalculated under the hood.
+
+        .. seelalso::
+
+            :meth:`munin.session.Session.insert`
+
+        .. warning::
+
+            Attention! The returned uid is the same as before, but the underlying
+            song is different. Do not compare by reference!
+
+        :param song: The song to modify, either an uid or a :class:`munin.song.Song`
+        :param sub_value_mapping: A mapping of attributes in values.
+        :returns: ``song.uid``
+        """
+        song = song_or_uid(self.database, song)
+        return self.database.modify(song, sub_value_mapping)
+
     def insert(self, value_mapping):
         """Insert a song without triggering a rebuild.
 
@@ -539,6 +596,7 @@ class Session:
         :returns: Number of times this song was played.
         :rtype: int
         """
+        song = song_or_uid(self.database, song)
         return self.database.playcount(song)
 
     def playcounts(self, n=0):
@@ -611,6 +669,10 @@ class Session:
     def listen_history(self):
         return self.database._listen_history
 
+    @property
+    def recom_history(self):
+        return self._recom_history
+
 
 if __name__ == '__main__':
     import unittest
